<?php
/**
 * @file
 * Admin callbacks for the Small Hay module.
 */

/**
 * Settings Form Callback.
 *
 * @param array $form
 * @param array $form_state
 *
 * @return array
 */
function smallhay_settings_form($form, &$form_state) {
  $form['api'] = array(
    '#type' => 'fieldset',
    '#title' => t('API Settings'),
    '#collapsible' => TRUE,
    '#description' => t(
      'Small Hay API Credentials need to be configured in the settings.php file. See Help or the README file for further instructions.'
    ),
  );
  $form['api']['test_api'] = array(
    '#type' => 'button',
    '#value' => t('Validate API Credentials'),
    '#executes_submit_callback' => FALSE,
    '#validate' => array('smallhay_api_credentials_validate'),
  );

  $form['admin'] = array(
    '#type' => 'fieldset',
    '#title' => t('Admin Settings'),
    '#collapsible' => TRUE,
  );
  $form['admin']['smallhay_disable_on_all'] = array(
    '#type' => 'checkbox',
    '#title' => t('Disable Small Hay on all pages.'),
    '#default_value' => variable_get('smallhay_disable_on_all'),
  );
  $form['admin']['smallhay_disable_on_admin'] = array(
    '#type' => 'checkbox',
    '#title' => t('Disable Small Hay on admin pages.'),
    '#default_value' => variable_get('smallhay_disable_on_admin'),
  );

  $path_keys = module_invoke_all('smallhay_path_key_info');

  $form['path_key'] = array(
    '#type' => 'fieldset',
    '#title' => t('Additional Path Keys'),
    '#collapsible' => TRUE,
  );
  foreach ($path_keys as $key => $data) {
    $form['path_key']['smallhay_path_key_' . $key] = array(
      '#type' => 'checkbox',
      '#title' => t($data['label']),
      '#description' => t($data['description']),
      '#default_value' => variable_get('smallhay_path_key_' . $key),
    );
  }

  return system_settings_form($form);
}

/**
 * API Credentials Validate Callback.
 *
 * @param array $form
 * @param array $form_state
 */
function smallhay_api_credentials_validate($form, &$form_state) {
  libraries_load('smallhay');
  $smallhay = new \SmallHay\API(variable_get('smallhay_client_id'), variable_get('smallhay_client_secret'));
  if ($smallhay->get_auth()) {
    drupal_set_message(t('API connection made successfully.'), 'status');
  }
  else {
    form_set_error('api', t('API Authorization failed.'));
  }
}

/**
 * Pages Form Callback.
 *
 * @param array $form
 * @param array $form_state
 *
 * @return array
 */
function smallhay_pages_form($form, &$form_state) {
  drupal_set_title(t('Small Hay - Pages'));

  $pages = smallhay_select_pages(TRUE);
  $rows = array();
  foreach ($pages as $page) {

    // Figure out path info.
    $path = explode(':', $page->path);
    $path = end($path);
    $path_text = $path;
    $path = ($path == variable_get('site_frontpage', 'node')) ? '<front>' : $path;

    $rows[$page->id] = array(
      'page' => l($path_text, $path),
      'page_key' => $page->path,
      'assets' => $page->asset_count,
      'status' => smallhay_determine_status(smallhay_page_status($page->id)),
      'view' => l('View assets', 'admin/config/development/smallhay/pages/' . $page->id),
    );
  }

  $form['pages'] = array(
    '#type' => 'tableselect',
    '#header' => array(
      'page' => t('Page'),
      'page_key' => t('Page Key'),
      'assets' => t('Assets'),
      'status' => t('Status'),
      'view' => t('View assets'),
    ),
    '#options' => $rows,
    '#empty' => t('No pages have been added.'),
    '#sticky' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Refresh selected pages'),
  );

  return $form;
}

/**
 * Pages Form Submit Callback.
 *
 * @param array $form
 * @param array $form_state
 */
function smallhay_pages_form_submit($form, $form_state) {
  $page_ids = array_filter($form_state['values']['pages']);

  // Need to remove data from the table.
  if (count($page_ids)) {
    $remote_ids = smallhay_select_pages(FALSE, array('id', 'remote_id'), array('id' => $page_ids));
    $remote_ids = array_filter(array_column($remote_ids, 'remote_id', 'id'));

    // Need to remove data from the API.
    if (count($remote_ids)) {
      libraries_load('smallhay');
      $smallhay = new \SmallHay\API(variable_get('smallhay_client_id'), variable_get('smallhay_client_secret'));
      // @todo - API has a limit of 100 at a time. Need to impose that limit here.
      $smallhay->delete_pages(json_encode(array_values($remote_ids)));
    }

    smallhay_delete_pages($page_ids);
  }
}

/**
 * Pages Filter Form Callback.
 *
 * @param array $form
 * @param array $form_state
 *
 * @return array
 */
function smallhay_pages_filter_form($form, &$form_state) {
  $form['wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter Assets'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['wrapper']['status'] = array(
    '#type' => 'select',
    '#title' => t('Status'),
    '#options' => array(
      '0' => t('All'),
      SMALLHAY_STATUS_CREATED => t('Created'),
      SMALLHAY_STATUS_UPLOADED => t('Uploaded'),
      SMALLHAY_STATUS_COMPLETE => t('Completed'),
    ),
    '#default_value' => (isset($_SESSION['smallhay']['pages']['filter']['status']) ? $_SESSION['smallhay']['pages']['filter']['status'] : 0),
  );
  $form['wrapper']['data_type'] = array(
    '#type' => 'select',
    '#title' => t('Asset Type'),
    '#options' => array(
      '0' => t('All'),
      SMALLHAY_DATA_TYPE_JAVASCRIPT => t('Javascript'),
    ),
    '#default_value' => (isset($_SESSION['smallhay']['pages']['filter']['data_type']) ? $_SESSION['smallhay']['pages']['filter']['data_type'] : 0),
  );
  $form['wrapper']['type'] = array(
    '#type' => 'select',
    '#title' => t('Type'),
    '#options' => array(
      '0' => t('All'),
      SMALLHAY_TYPE_FILE => t('File'),
      SMALLHAY_TYPE_RAW => t('Raw'),
    ),
    '#default_value' => (isset($_SESSION['smallhay']['pages']['filter']['type']) ? $_SESSION['smallhay']['pages']['filter']['type'] : 0),
  );
  $form['wrapper']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
  );
  $form['wrapper']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset'),
  );

  return $form;
}

/**
 * Pages Filter Form Submit Callback.
 *
 * @param array $form
 * @param array $form_state
 */
function smallhay_pages_filter_form_submit($form, $form_state) {
  if ($form_state['values']['op'] == t('Filter')) {
    $_SESSION['smallhay']['pages']['filter']['status'] = $form_state['values']['status'];
    $_SESSION['smallhay']['pages']['filter']['data_type'] = $form_state['values']['data_type'];
    $_SESSION['smallhay']['pages']['filter']['type'] = $form_state['values']['type'];
  }
  else {
    $_SESSION['smallhay']['pages']['filter']['status'] = 0;
    $_SESSION['smallhay']['pages']['filter']['data_type'] = 0;
    $_SESSION['smallhay']['pages']['filter']['type'] = 0;
  }
}

/**
 * Page Assets Callback.
 *
 * @param int $page_id
 *
 * @return array
 * @throws \Exception
 */
function smallhay_page_assets($page_id) {
  $build = array();
  $page = smallhay_select_page('id', $page_id);
  drupal_set_title(t('Small Hay - Page Assets for %page', array('%page' => $page->path)), PASS_THROUGH);

  $assets = smallhay_select_page_assets(array('data_type', 'name', 'input', 'type', 'status'), array('id' => $page_id));
  $rows = array();
  foreach ($assets as $asset) {

    // Skip if filter status does not match the asset status.
    if (isset($_SESSION['smallhay']['pages']['filter']['status'])) {
      if ($_SESSION['smallhay']['pages']['filter']['status'] != '0' && $_SESSION['smallhay']['pages']['filter']['status'] != $asset->status) {
        continue;
      }
    }

    // Skip if filter data type does not match the asset type.
    if (isset($_SESSION['smallhay']['pages']['filter']['data_type'])) {
      if ($_SESSION['smallhay']['pages']['filter']['data_type'] != '0' && $_SESSION['smallhay']['pages']['filter']['data_type'] != $asset->data_type) {
        continue;
      }
    }

    // Skip if filter type does not match the type.
    if (isset($_SESSION['smallhay']['pages']['filter']['data_type'])) {
      if ($_SESSION['smallhay']['pages']['filter']['type'] != '0' && $_SESSION['smallhay']['pages']['filter']['type'] != $asset->type) {
        continue;
      }
    }

    // Determine data type.
    switch ($asset->data_type) {
      case SMALLHAY_DATA_TYPE_JAVASCRIPT:
        $data_type = t('Javascript');
        break;
    }

    // Determine type.
    switch ($asset->type) {
      case SMALLHAY_TYPE_FILE:
        $type = t('File');
        break;

      case SMALLHAY_TYPE_RAW:
        $type = t('Raw');
        break;
    }

    $preview = htmlentities($asset->input);
    if (strlen($asset->input) > 153) {
      $preview = substr($preview, 0, 150) . '...';
    }

    $rows[$asset->name] = array(
      'asset' => $asset->name,
      'data_type' => $data_type,
      'preview' => $preview,
      'type' => $type,
      'status' => smallhay_determine_status($asset->status),
      'view' => l('View', 'admin/config/development/smallhay/pages/' . $page_id . '/' . $asset->name),
    );
  }

  $build['filter'] = drupal_get_form('smallhay_pages_filter_form');
  $build['table'] = array(
    '#theme' => 'table',
    '#header' => array(
      'asset' => t('Asset ID'),
      'data_type' => t('Data Type'),
      'preview' => t('File Preview'),
      'type' => t('Type'),
      'status' => t('Status'),
      'view' => t('View'),
    ),
    '#rows' => $rows,
    '#empty' => t('No assets have been added.'),
    '#sticky' => TRUE,
  );

  return $build;
}

/**
 * Page Asset Callback.
 *
 * @param int $page_id
 * @param string $asset_name
 *
 * @return array
 * @throws \Exception
 */
function smallhay_page_asset($page_id, $asset_name) {
  $page = smallhay_select_page('id', $page_id);
  drupal_set_title(t('Small Hay - Page Asset %asset for %page', array('%asset' => $asset_name, '%page' => $page->path)), PASS_THROUGH);
  $asset = smallhay_select_page_asset($page_id, $asset_name);

  return array(
    '#type' => 'container',
    'input' => array(
      '#type' => 'fieldset',
      '#title' => t('Preview'),
      'data' => array(
        '#markup' => '<pre>' . htmlentities($asset->input) . '</pre>',
      ),
    ),
  );
}

/**
 * Determine Status.
 * 
 * Return the status as a string.
 * 
 * @param int status
 * 
 * @return string
 */
function smallhay_determine_status($status) {
  $status_string = '';

  // Determine status.
  switch ($status) {
    case SMALLHAY_STATUS_UPLOADED:
      $status_string = t('Uploaded');
      break;

    case SMALLHAY_STATUS_COMPLETE:
      $status_string = t('Complete');
      break;

    case SMALLHAY_STATUS_CREATED:
    default:
      $status_string = t('Created');
      break;
  }

  return $status_string;
}
